@@ -23,6 +23,8 @@
 #include <config.h>
 
 #include <gtk/gtk.h>
+#include <glib.h>
+#include <glib/gprintf.h>
 #include <glib/gi18n.h>
 #include <string.h>
 
@@ -37,11 +39,10 @@ create_model (void)
 {
 	GtkTreeStore *mdl = gtk_tree_store_new (NUM_TREE_COLUMNS,
 						G_TYPE_STRING,	/* COL_DIR_NAME */
-						G_TYPE_STRING,	/* COL_H_FULLPATH */
-						G_TYPE_FLOAT,	/* COL_H_PERC */
+						G_TYPE_STRING,	/* COL_H_PARSENAME */
+						G_TYPE_DOUBLE,	/* COL_H_PERC */
 						G_TYPE_STRING,	/* COL_DIR_SIZE */
 						G_TYPE_UINT64,	/* COL_H_SIZE */
-						G_TYPE_STRING,	/* COL_PERC */
 						G_TYPE_UINT64,	/* COL_H_ALLOCSIZE */
 						G_TYPE_STRING,	/* COL_ELEMENTS */
 						G_TYPE_INT,	/* COL_H_ELEMENTS */
@@ -49,11 +50,6 @@ create_model (void)
 						G_TYPE_UINT64	/* COL_H_HARDLINK */
 						);
 
-	/* Defaults to sort-by-size */
-	gtk_tree_sortable_set_sort_column_id ((GtkTreeSortable *) mdl,
-					      COL_H_SIZE,
-					      GTK_SORT_DESCENDING);
-
 	return mdl;
 }
 
@@ -69,32 +65,16 @@ on_tv_row_expanded (GtkTreeView *treeview,
 static void
 on_tv_cur_changed (GtkTreeView *treeview, gpointer data)
 {
-
 	GtkTreeIter iter;
-	gchar *text = NULL;
+	gchar *parsename = NULL;
 
 	gtk_tree_selection_get_selected (gtk_tree_view_get_selection (treeview), NULL, &iter);
 
 	if (gtk_tree_store_iter_is_valid (baobab.model, &iter)) {
-			gtk_tree_model_get (GTK_TREE_MODEL (baobab.model), &iter,
-				    COL_H_FULLPATH, &text, -1);
+		gtk_tree_model_get (GTK_TREE_MODEL (baobab.model), &iter,
+				    COL_H_PARSENAME, &parsename, -1);
 	}
-	
-	set_glade_widget_sens("menu_treemap",FALSE);
-	if (text) {
-		gchar *msg;
-
-		/* make sure it is utf8 */
-		msg = g_filename_display_name (text);
-
-		set_statusbar (msg);
-		if (strcmp (text, "") != 0 )
-			set_glade_widget_sens("menu_treemap",TRUE);
-
-		g_free (msg);
-		g_free (text);
 	}
-}
 
 static gboolean
 on_tv_button_press (GtkWidget *widget,
@@ -103,8 +83,7 @@ on_tv_button_press (GtkWidget *widget,
 {
 	GtkTreePath *path;
 	GtkTreeIter iter;
-	gchar *trash_path, *dir_path;
-	gboolean is_trash = FALSE;
+	
 
 	if (baobab.CONTENTS_CHANGED_DELAYED) {
 		baobab.CONTENTS_CHANGED_DELAYED = FALSE;
@@ -128,26 +107,19 @@ on_tv_button_press (GtkWidget *widget,
 	gtk_tree_model_get_iter (GTK_TREE_MODEL (baobab.model), &iter,
 				 path);
 	gtk_tree_model_get (GTK_TREE_MODEL (baobab.model), &iter,
-			    COL_H_FULLPATH, &baobab.selected_path, -1);
+			    COL_H_PARSENAME, &baobab.selected_path, -1);
 	
-	if (strcmp (baobab.selected_path, "") == 0) {
-		set_glade_widget_sens("menu_treemap",FALSE);
-		gtk_tree_path_free (path);
-		return FALSE;
-	}
-
 	/* right-click */
 	if (event->button == 3) {
-		trash_path = get_trash_path(baobab.selected_path);
-		dir_path = g_path_get_dirname(baobab.selected_path);
-		if (trash_path)
-			if (strcmp(trash_path, dir_path)==0)
-				is_trash = TRUE;
-		popupmenu_list (path, event, is_trash);
+		GFile *file;
+
+		file = g_file_parse_name (baobab.selected_path);
+
+		popupmenu_list (path, event, can_trash_file (file));
 
 		gtk_tree_path_free (path);
-		g_free(trash_path);
-		g_free(dir_path);
+		g_object_unref (file);
+
 		return FALSE;
 	}
 
@@ -187,6 +159,28 @@ baobab_treeview_equal_func (GtkTreeModel *model,
 	return results;
 }
 
+static void
+perc_cell_data_func (GtkTreeViewColumn *col,
+		     GtkCellRenderer *renderer,
+		     GtkTreeModel *model,
+		     GtkTreeIter *iter,
+		     gpointer user_data)
+{
+	gdouble perc;
+	gchar textperc[10];
+
+	gtk_tree_model_get (model, iter, COL_H_PERC, &perc, -1);
+
+ 	if (perc < 0)
+		strcpy (textperc, "-.- %");
+	else if (perc == 100.0)
+		strcpy (textperc, "100 %");
+	else
+ 		g_sprintf (textperc, " %.1f %%", perc);
+
+	g_object_set (renderer, "text", textperc, NULL);
+}
+
 GtkWidget *
 create_directory_treeview (void)
 {
@@ -227,9 +221,9 @@ create_directory_treeview (void)
 
 	cell = gtk_cell_renderer_text_new ();
 	gtk_tree_view_column_pack_start (col, cell, TRUE);
-	gtk_tree_view_column_set_attributes (col, cell, "markup",
-	                                     COL_PERC, "text",
-	                                     COL_PERC, NULL);
+	gtk_tree_view_column_set_cell_data_func (col, cell,
+						 perc_cell_data_func,
+						 NULL, NULL);
 
 	g_object_set (G_OBJECT (cell), "xalign", (gfloat) 1.0, NULL);
 	gtk_tree_view_column_set_sort_column_id (col, COL_H_PERC);
@@ -245,13 +239,17 @@ create_directory_treeview (void)
 							COL_DIR_SIZE, "text",
 							COL_DIR_SIZE, NULL);
 	g_object_set (G_OBJECT (cell), "xalign", (gfloat) 1.0, NULL);
-	gtk_tree_view_column_set_sort_column_id (col, COL_H_SIZE);
+	gtk_tree_view_column_set_sort_column_id (col,
+						 baobab.show_allocated ? COL_H_ALLOCSIZE : COL_H_SIZE);
 	gtk_tree_view_column_set_reorderable (col, TRUE);
 	gtk_tree_view_column_set_title (col, _("Size"));
 	gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 	gtk_tree_view_column_set_resizable (col, TRUE);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (tvw), col);
 
+	/* store this column, we need it when toggling 'allocated' */
+	g_object_set_data (G_OBJECT (tvw), "baobab_size_col", col);
+
 	/* objects column */
 	cell = gtk_cell_renderer_text_new ();
 	col = gtk_tree_view_column_new_with_attributes (NULL, cell, "markup",
@@ -285,9 +283,41 @@ create_directory_treeview (void)
 
 	baobab.model = create_model ();
 
+	/* By default, sort by size */
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (baobab.model),
+					      baobab.show_allocated ? COL_H_ALLOCSIZE : COL_H_SIZE,
+					      GTK_SORT_DESCENDING);
+
 	gtk_tree_view_set_model (GTK_TREE_VIEW (tvw),
 				 GTK_TREE_MODEL (baobab.model));
 	g_object_unref (baobab.model);
 
 	return tvw;
 }
+
+void
+baobab_treeview_show_allocated_size (GtkWidget *tv,
+				     gboolean show_allocated)
+{
+	gint sort_id;
+	gint new_sort_id;
+	GtkSortType order;
+	GtkTreeViewColumn *size_col;
+
+	g_return_if_fail (GTK_IS_TREE_VIEW (tv));
+
+	gtk_tree_sortable_get_sort_column_id (GTK_TREE_SORTABLE (baobab.model),
+					      &sort_id, &order);
+
+	/* set the sort id for the size column */
+	size_col = g_object_get_data (G_OBJECT (tv), "baobab_size_col");
+	new_sort_id = show_allocated ? COL_H_ALLOCSIZE : COL_H_SIZE;
+	gtk_tree_view_column_set_sort_column_id (size_col, new_sort_id);
+
+	/* if we are currently sorted on size or allocated size,
+	 * then trigger a resort (with the same order) */
+	if (sort_id == COL_H_SIZE || sort_id == COL_H_ALLOCSIZE) {
+		gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (baobab.model),
+						      new_sort_id, order);
+	}
+}
