@@ -22,16 +22,11 @@
  
 #include <config.h>
 
-#include <pwd.h>
-#include <time.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
-#include <glib/gprintf.h>
+#include <gio/gio.h>
 #include <libgnomeui/libgnomeui.h>
 #include <libgnomeui/gnome-ui-init.h>
-#include <libgnomevfs/gnome-vfs.h>
-#include <libgnomevfs/gnome-vfs-mime.h>
-#include <libgnomevfs/gnome-vfs-mime-handlers.h>
 #include <gconf/gconf-client.h>
 #include <glibtop.h>
 
@@ -44,10 +39,11 @@
 
 #include "gedit-spinner.h"
 
-static GnomeVFSMonitorHandle *handle_mtab;
-static GnomeVFSMonitorHandle *handle_home;
+#include "baobab-treemap.h"
+#include "baobab-ringschart.h"
 
-static void check_UTF (GString *);
+static GVolumeMonitor 	 *monitor_vol;
+static GFileMonitor	 *monitor_home;
 
 static void push_iter_in_stack (GtkTreeIter *);
 static GtkTreeIter pop_iter_from_stack (void);
@@ -57,72 +53,125 @@ static GtkTreeIter currentiter;
 static GtkTreeIter firstiter;
 static GQueue *iterstack = NULL;
 
+static GFile *current_location = NULL;
+
 #define BUSY_IMAGE_PATH		BAOBAB_PIX_DIR "busy.gif"
 #define DONE_IMAGE_PATH		BAOBAB_PIX_DIR "done.png"
 
 
 static gboolean
-scan_is_local (const gchar *uri_dir)
+scan_is_local (GFile	*file)
 {
-	GnomeVFSFileInfo *info;
-	GnomeVFSResult res;
-	gboolean ret;
+	gchar *uri_scheme;
+	gboolean ret = FALSE;
 
-	info = gnome_vfs_file_info_new ();
-	res = gnome_vfs_get_file_info (uri_dir,
-				       info, GNOME_VFS_FILE_INFO_DEFAULT);
+	uri_scheme = g_file_get_uri_scheme (file);
+	if (g_ascii_strcasecmp(uri_scheme,"file") == 0)
+		ret = TRUE;
 
-	ret = ((res == GNOME_VFS_OK) &&
-	       (info->flags & GNOME_VFS_FILE_FLAGS_LOCAL));
-
-	gnome_vfs_file_info_unref (info);
+	g_free (uri_scheme);
 
 	return ret;
 }
 
 void
-set_busy (gboolean busy)
+baobab_set_busy (gboolean busy)
 {
+	static GdkCursor *busy_cursor = NULL;
+	GdkCursor *cursor = NULL;
+
 	if (busy == TRUE) {
+		if (!busy_cursor) {
+			busy_cursor = gdk_cursor_new (GDK_WATCH);
+		}
+		cursor = busy_cursor;
+
 		gedit_spinner_start (GEDIT_SPINNER (baobab.spinner));
- 		baobab_ringschart_freeze_updates (baobab.ringschart);
-		baobab_ringschart_set_init_depth (baobab.ringschart, 0);
+
+		baobab_chart_freeze_updates (baobab.rings_chart);
+		baobab_chart_set_summary_mode (baobab.rings_chart, FALSE);
+
+		baobab_chart_freeze_updates (baobab.treemap_chart);
+		baobab_chart_set_summary_mode (baobab.treemap_chart, FALSE);
+
+		gtk_widget_set_sensitive (baobab.chart_type_combo, FALSE);
 	}
 	else {
 		gedit_spinner_stop (GEDIT_SPINNER (baobab.spinner));
- 		baobab_ringschart_thaw_updates (baobab.ringschart);
+
+		baobab_chart_thaw_updates (baobab.rings_chart);
+		baobab_chart_thaw_updates (baobab.treemap_chart);
+
+		gtk_widget_set_sensitive (baobab.chart_type_combo, TRUE);
+	}
+
+	/* change the cursor */
+	if (baobab.window->window) {
+		gdk_window_set_cursor (baobab.window->window, cursor);
 	}
 }
 
-/*
- * start scanning on a specific directory
- */
+/* menu & toolbar sensitivity */
+static void
+check_menu_sens (gboolean scanning)
+{
+	if (scanning) {
+
+		while (gtk_events_pending ())
+			gtk_main_iteration ();
+
+		set_statusbar (_("Scanning..."));
+		set_glade_widget_sens ("expand_all", TRUE);
+		set_glade_widget_sens ("collapse_all", TRUE);
+	}
+
+	set_glade_widget_sens ("tbscanhome", !scanning);
+	set_glade_widget_sens ("tbscanall", !scanning);
+	set_glade_widget_sens ("tbscandir", !scanning);
+	set_glade_widget_sens ("menuscanhome", !scanning);
+	set_glade_widget_sens ("menuallfs", !scanning);
+	set_glade_widget_sens ("menuscandir", !scanning);
+	set_glade_widget_sens ("tbstop", scanning);
+	set_glade_widget_sens ("tbrescan", !scanning && current_location != NULL);
+	set_glade_widget_sens ("menustop", scanning);
+	set_glade_widget_sens ("menurescan", !scanning && current_location != NULL);
+	set_glade_widget_sens ("preferenze1", !scanning);
+	set_glade_widget_sens ("menu_scan_rem", !scanning);
+	set_glade_widget_sens ("tb_scan_remote", !scanning);
+	set_glade_widget_sens ("ck_allocated",
+			       !scanning &&
+			       baobab.is_local && !g_noactivescans);
+}
+
 void
-start_proc_on_dir (const gchar *dir)
+baobab_scan_location (GFile *file)
 {
-	GdkCursor *cursor = NULL;
 	GtkWidget *ck_allocated;
 
-	if (!baobab_check_dir (dir))
+	if (!baobab_check_dir (file))
 		return;
-		
+
 	if (iterstack !=NULL)
 		return;
-		
-	g_string_assign (baobab.last_scan_command, dir);
+
+	if (current_location)
+		g_object_unref (current_location);
+	current_location = g_object_ref (file);
+
+	g_noactivescans = FALSE; 
 	baobab.STOP_SCANNING = FALSE;
-	set_busy (TRUE);
+	baobab_set_busy (TRUE);
 	check_menu_sens (TRUE);
 	gtk_tree_store_clear (baobab.model);
 	currentdepth = -1;	/* flag */
 	iterstack = g_queue_new ();
 
 	/* check if the file system is local or remote */
-	baobab.is_local = scan_is_local (dir);
+	baobab.is_local = scan_is_local (file);
 	ck_allocated = glade_xml_get_widget (baobab.main_xml, "ck_allocated");
 	if (!baobab.is_local) {
-		gtk_toggle_button_set_active ((GtkToggleButton *)
-					      ck_allocated, FALSE);
+		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (ck_allocated),
+						FALSE);
 		gtk_widget_set_sensitive (ck_allocated, FALSE);
 		baobab.show_allocated = FALSE;
 	}
@@ -130,30 +179,21 @@ start_proc_on_dir (const gchar *dir)
 		gtk_widget_set_sensitive (ck_allocated, TRUE);
 	}
 
-	getDir (dir);
-
-	/* change the cursor */
-	if (baobab.window->window) {
-		cursor = gdk_cursor_new (GDK_WATCH);
-		gdk_window_set_cursor (baobab.window->window, cursor);
-	}
+	baobab_scan_execute (file);
 
 	/* set statusbar, percentage and allocated/normal size */
 	set_statusbar (_("Calculating percentage bars..."));
 	gtk_tree_model_foreach (GTK_TREE_MODEL (baobab.model),
 				show_bars,
 				NULL);
-	set_busy (FALSE);
+
+	baobab_chart_set_max_depth (baobab.rings_chart, baobab.model_max_depth);
+	baobab_chart_set_max_depth (baobab.treemap_chart, baobab.model_max_depth);
+
+	baobab_set_busy (FALSE);
 	check_menu_sens (FALSE);
 	set_statusbar (_("Ready"));
 
-	/* cursor clean up */
-	if (baobab.window->window) {
-		gdk_window_set_cursor (baobab.window->window, NULL);
-		if (cursor)
-			gdk_cursor_unref (cursor);
-	}
-
 	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (baobab.tree_view));
 	baobab.STOP_SCANNING = TRUE;
 	g_queue_free (iterstack);
@@ -162,11 +202,48 @@ start_proc_on_dir (const gchar *dir)
 }
 
 void
-rescan_current_dir (void)
+baobab_scan_home (void)
 {
-	g_return_if_fail (baobab.last_scan_command != NULL);
+	GFile *file;
 
-	start_proc_on_dir (baobab.last_scan_command->str);
+	file = g_file_new_for_path (g_get_home_dir ());
+	baobab_scan_location (file);
+	g_object_unref (file);
+}
+
+void
+baobab_scan_root (void)
+{
+	GFile *file;
+
+	file = g_file_new_for_uri ("file:///");
+	baobab_scan_location (file);
+	g_object_unref (file);
+}
+
+void
+baobab_rescan_current_dir (void)
+{
+	g_return_if_fail (current_location != NULL);
+
+	baobab_get_filesystem (&g_fs);
+	set_label_scan (&g_fs);
+	show_label ();
+
+	g_object_ref (current_location);
+	baobab_scan_location (current_location);
+	g_object_unref (current_location);
+}
+
+void
+baobab_stop_scan (void)
+{
+	baobab.STOP_SCANNING = TRUE;
+
+	set_statusbar (_("Calculating percentage bars..."));
+	gtk_tree_model_foreach (GTK_TREE_MODEL (baobab.model),
+				show_bars, NULL);
+	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (baobab.tree_view));
 }
 
 /*
@@ -175,12 +252,9 @@ rescan_current_dir (void)
 static void
 prefill_model (struct chan_data *data)
 {
-	GString *cdir;
-	char *basename;
 	GtkTreeIter iter, iterparent;
-	gchar *str;
-
-	cdir = g_string_new ("");
+	char *name;
+	char *str;
 
 	if (currentdepth == -1) {
 		gtk_tree_store_append (baobab.model, &iter, NULL);
@@ -216,30 +290,27 @@ prefill_model (struct chan_data *data)
 		}
 		gtk_tree_store_append (baobab.model, &iter, &tempiter);
 	}
+
 	currentdepth = data->depth;
 	push_iter_in_stack (&iter);
 	currentiter = iter;
 
-	basename = g_filename_display_basename (data->dir);
-	g_string_assign (cdir, basename);
-	g_free (basename);
-
-	/* check UTF-8 and locale */
-	check_UTF (cdir);
+	/* in case filenames contains gmarkup */
+	name = g_markup_escape_text (data->display_name, -1);
 
 	str = g_strdup_printf ("<small><i>%s</i></small>", _("Scanning..."));
 
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (baobab.tree_view), TRUE);
 	gtk_tree_store_set (baobab.model, &iter,
-			    COL_DIR_NAME, cdir->str,
-			    COL_H_FULLPATH, "",
+			    COL_DIR_NAME, name,
+			    COL_H_PARSENAME, "",
 			    COL_H_ELEMENTS, -1, 
  			    COL_H_PERC, -1.0,
 			    COL_DIR_SIZE, str,
 			    COL_ELEMENTS, str, -1);
 
-	g_string_free (cdir, TRUE);
-	g_free(str);
+	g_free (name);
+	g_free (str);
 
 	while (gtk_events_pending ()) {
 		gtk_main_iteration ();
@@ -258,14 +329,14 @@ first_row (void)
 
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (baobab.tree_view), FALSE);
 	gtk_tree_store_append (baobab.model, &firstiter, NULL);
-	size = gnome_vfs_format_file_size_for_display (g_fs.used);
-	g_assert(g_fs.total != 0);
+	size = g_format_size_for_display (g_fs.used);
+	g_assert (g_fs.total != 0);
 	perc = ((gdouble) g_fs.used * 100) / (gdouble) g_fs.total;
 
 	label = g_strdup_printf ("<i>%s</i>", _("Total filesystem usage:"));
 	gtk_tree_store_set (baobab.model, &firstiter,
 			    COL_DIR_NAME, label,
-			    COL_H_FULLPATH, "",
+			    COL_H_PARSENAME, "",
 			    COL_H_PERC, perc,
 			    COL_DIR_SIZE, size,
 			    COL_H_SIZE, g_fs.used,
@@ -282,29 +353,24 @@ fill_model (struct chan_data *data)
 {
 	GtkTreeIter iter;
 	GString *hardlinks;
-	GString *basename;
-	GString *elementi;
+	GString *elements;
+	char *name;
 	char *size;
 	char *alloc_size;
-	char *basename_cstr;
 
 	if (data->elements == -1) {
 		prefill_model (data);
 		return;
 	}
 
-	basename = g_string_new ("");
-	basename_cstr = g_filename_display_basename (data->dir);
-	g_string_assign (basename, basename_cstr);
-	g_free (basename_cstr);
-
-	check_UTF (basename);
-
 	iter = pop_iter_from_stack ();
 
+	/* in case filenames contains gmarkup */
+	name = g_markup_escape_text (data->display_name, -1);
+
 	hardlinks = g_string_new ("");
 	if (data->tempHLsize > 0) {
-		size = gnome_vfs_format_file_size_for_display (data->tempHLsize);
+		size = g_format_size_for_display (data->tempHLsize);
 		g_string_assign (hardlinks, "<i>(");
 		g_string_append (hardlinks, _("contains hardlinks for:"));
 		g_string_append (hardlinks, " ");
@@ -313,22 +379,22 @@ fill_model (struct chan_data *data)
 		g_free (size);
 	}
 
-	elementi = g_string_new ("");
-	g_string_printf (elementi,
+	elements = g_string_new ("");
+	g_string_printf (elements,
 			 ngettext ("% 5d item", "% 5d items",
 				   data->elements), data->elements);
 
-	size = gnome_vfs_format_file_size_for_display (data->size);
-	alloc_size = gnome_vfs_format_file_size_for_display (data->alloc_size);
+	size = g_format_size_for_display (data->size);
+	alloc_size = g_format_size_for_display (data->alloc_size);
 
 	gtk_tree_store_set (baobab.model, &iter,
-			    COL_DIR_NAME, basename->str,
-			    COL_H_FULLPATH, data->dir,
-			    COL_H_PERC, -1.0, 
+			    COL_DIR_NAME, name,
+			    COL_H_PARSENAME, data->parse_name,
+			    COL_H_PERC, -1.0,
 			    COL_DIR_SIZE,
 			    baobab.show_allocated ? alloc_size : size,
 			    COL_H_SIZE, data->size,
-			    COL_ELEMENTS, elementi->str,
+			    COL_ELEMENTS, elements->str,
 			    COL_H_ELEMENTS, data->elements,
 			    COL_HARDLINK, hardlinks->str,
 			    COL_H_HARDLINK, data->tempHLsize,
@@ -339,8 +405,8 @@ fill_model (struct chan_data *data)
 	}
 
 	g_string_free (hardlinks, TRUE);
-	g_string_free (basename, TRUE);
-	g_string_free (elementi, TRUE);
+	g_string_free (elements, TRUE);
+	g_free (name);
 	g_free (size);
 	g_free (alloc_size);
 }
@@ -368,57 +434,33 @@ pop_iter_from_stack (void)
 }
 
 void
-check_UTF (GString *name)
+baobab_set_excluded_locations (GSList *excluded_uris)
 {
-	char *str;
-	char *escaped_str;
+	GSList *l;
 
-	str = g_filename_to_utf8 (name->str, -1, NULL, NULL, NULL);
-
-	if (!str) {
-		str = g_locale_to_utf8 (name->str, -1, NULL, NULL, NULL);
-
-		if (!str) {
-			str = g_strjoin ("<i>",
-					 _("Invalid UTF-8 characters"),
-					 "</i>", NULL);
-		}
+	g_slist_foreach (baobab.excluded_locations, (GFunc) g_object_unref, NULL);
+	g_slist_free (baobab.excluded_locations);
+	baobab.excluded_locations = NULL;
+	for (l = excluded_uris; l != NULL; l = l->next) {
+		baobab.excluded_locations = g_slist_prepend (baobab.excluded_locations,
+						g_file_new_for_uri (l->data));
 	}
-
-	g_assert (str);
-
-	escaped_str = g_markup_escape_text (str, strlen (str));
-
-	g_string_assign (name, escaped_str);
-
-	g_free (str);
-	g_free (escaped_str);
-}
-
-gint
-list_find (gconstpointer a, gconstpointer b)
-{
-	gchar *str_a, *str_b;
-	gint ret;
-
-	str_a = gnome_vfs_format_uri_for_display (a);
-	str_b = gnome_vfs_format_uri_for_display (b);
-
-	ret = strcmp (str_a, str_b);
-
-	g_free (str_a);
-	g_free (str_b);
-
-	return ret;
 }
 
 gboolean
-is_excluded_dir (const gchar *dir)
+baobab_is_excluded_location (GFile *file)
 {
-	g_return_if_fail (dir != NULL);
+	gboolean ret = FALSE;
+	GSList *l;
 
-	return (baobab.bbExcludedDirs &&
-		(g_slist_find_custom (baobab.bbExcludedDirs, dir, list_find) != NULL));
+	g_return_val_if_fail (file != NULL, FALSE);
+
+	for (l=baobab.excluded_locations; l != NULL; l = l->next) {	
+		if (g_file_equal(l->data, file))
+			ret = TRUE;
+	}
+
+	return ret;
 }
 
 void
@@ -532,20 +574,20 @@ baobab_create_toolbar (void)
 
 	separator = gtk_separator_tool_item_new ();
 	gtk_separator_tool_item_set_draw (GTK_SEPARATOR_TOOL_ITEM (separator), FALSE);
-	gtk_tool_item_set_expand (GTK_TOOL_ITEM (separator), TRUE);	
+	gtk_tool_item_set_expand (GTK_TOOL_ITEM (separator), TRUE);
 	gtk_container_add (GTK_CONTAINER (toolbar), GTK_WIDGET (separator));
 	gtk_widget_show (GTK_WIDGET (separator));
 
 	baobab.spinner = gedit_spinner_new ();
-	item = gtk_tool_item_new ();		
+	item = gtk_tool_item_new ();
 	gtk_container_add (GTK_CONTAINER (item), baobab.spinner);
-  	gtk_container_add (GTK_CONTAINER (toolbar), GTK_WIDGET (item));
+	gtk_container_add (GTK_CONTAINER (toolbar), GTK_WIDGET (item));
 	gtk_widget_show (GTK_WIDGET (baobab.spinner));
 	gtk_widget_show (GTK_WIDGET (item));
 
 	g_signal_connect (item, "toolbar-reconfigured",
 			  G_CALLBACK (toolbar_reconfigured_cb), baobab.spinner);
-
+	toolbar_reconfigured_cb (item, GEDIT_SPINNER (baobab.spinner));
 	baobab_toolbar_style (NULL, 0, NULL, NULL);
 
 	visible = gconf_client_get_bool (baobab.gconf_client,
@@ -573,11 +615,56 @@ baobab_create_statusbar (void)
 	set_statusbar_visible (visible);
 }
 
+static void
+baobab_subfolderstips_toggled (GConfClient *client,
+			       guint cnxn_id,
+			       GConfEntry *entry,
+			       gpointer user_data)
+{
+	baobab_ringschart_set_subfoldertips_enabled (baobab.rings_chart,
+						     gconf_client_get_bool (baobab.gconf_client,
+									    BAOBAB_SUBFLSTIPS_VISIBLE_KEY,
+									    NULL));
+}
+
+static void
+sanity_check_excluded_locations (void)
+{
+	GFile *root;
+
+	/* Verify if gconf wrongly contains root dir exclusion, and remove it from gconf. */
+	root = g_file_new_for_uri ("file:///");
+	if (baobab_is_excluded_location (root)) {
+		GSList *uri_list, *l;
+
+		baobab.excluded_locations = g_slist_delete_link (baobab.excluded_locations, 
+						g_slist_find (baobab.excluded_locations, 
+							      root));
+
+		for (l = baobab.excluded_locations; l != NULL; l = l->next) {
+			uri_list = g_slist_prepend (uri_list, g_file_get_uri(l->data));
+		}
+		
+		gconf_client_set_list (baobab.gconf_client,
+				       PROPS_SCAN_KEY,
+				       GCONF_VALUE_STRING, 
+				       uri_list,
+				       NULL);
+
+		g_slist_foreach (uri_list, (GFunc) g_free, NULL);
+		g_slist_free (uri_list);
+	}
+
+	g_object_unref (root);
+}
+
 static void
 baobab_init (void)
 {
-	GnomeVFSResult result;
-	GnomeVFSVolumeMonitor *volmonitor;
+	GSList *uri_list;
+	GFile *file;
+	GError *error = NULL;
+	monitor_home = NULL;
 
 	/* Load Glade */
 	baobab.main_xml = glade_xml_new (BAOBAB_GLADE_FILE,
@@ -586,7 +673,6 @@ baobab_init (void)
 
 	/* Misc */
 	baobab.label_scan = NULL;
-	baobab.last_scan_command = g_string_new ("/");
 	baobab.CONTENTS_CHANGED_DELAYED = FALSE;
 	baobab.STOP_SCANNING = TRUE;
 	baobab.show_allocated = TRUE;
@@ -600,19 +686,21 @@ baobab_init (void)
 				 NULL, NULL, NULL);
 	gconf_client_notify_add (baobab.gconf_client, SYSTEM_TOOLBAR_STYLE, baobab_toolbar_style,
 				 NULL, NULL, NULL);				 
-	baobab.bbExcludedDirs = gconf_client_get_list (baobab.gconf_client, PROPS_SCAN_KEY,
-						       GCONF_VALUE_STRING, NULL);
-	
-	/* Verify if gconf wrongly contains root dir exclusion, and remove it from gconf. */
-	if (is_excluded_dir("/")) {
-		baobab.bbExcludedDirs = g_slist_delete_link (baobab.bbExcludedDirs, 
-						g_slist_find_custom(baobab.bbExcludedDirs, 
-							"/", list_find));
-		gconf_client_set_list (baobab.gconf_client, PROPS_SCAN_KEY,
-						GCONF_VALUE_STRING, 
-						baobab.bbExcludedDirs, NULL);
-	}
-	
+	gconf_client_notify_add (baobab.gconf_client, BAOBAB_SUBFLSTIPS_VISIBLE_KEY, baobab_subfolderstips_toggled,
+				 NULL, NULL, NULL);
+
+	uri_list = gconf_client_get_list (baobab.gconf_client,
+						      PROPS_SCAN_KEY,
+						      GCONF_VALUE_STRING,
+						      NULL);
+
+	baobab_set_excluded_locations (uri_list);
+
+	g_slist_foreach (uri_list, (GFunc) g_free, NULL);
+	g_slist_free (uri_list);
+
+	sanity_check_excluded_locations ();
+
 	baobab.bbEnableHomeMonitor = gconf_client_get_bool (baobab.gconf_client,
 							    PROPS_ENABLE_HOME_MONITOR,
 							    NULL);
@@ -621,26 +709,27 @@ baobab_init (void)
 
 	baobab_create_statusbar ();
 
-	/* start VFS monitoring */
-	handle_home = NULL;
-
-	volmonitor = gnome_vfs_get_volume_monitor ();
-	g_signal_connect (volmonitor, "volume_mounted",
-			  G_CALLBACK (volume_changed), NULL);
-	g_signal_connect (volmonitor, "volume_unmounted",
+	/* start monitoring */
+	monitor_vol = g_volume_monitor_get ();
+	g_signal_connect (monitor_vol, "volume_changed",
 			  G_CALLBACK (volume_changed), NULL);
 
-	result = gnome_vfs_monitor_add (&handle_home,
-					g_get_home_dir (),
-					GNOME_VFS_MONITOR_DIRECTORY,
-					contents_changed_cb, NULL);
+	file = g_file_new_for_path (g_get_home_dir ());
+	monitor_home = g_file_monitor_directory (file, 0, NULL, &error);
+	g_object_unref (file);
 
-	if (result != GNOME_VFS_OK) {
-		message (_("Could not initialize GNOME VFS monitoring"),
+	if (!monitor_home) {
+		message (_("Could not initialize monitoring"),
 			 _("Changes to your home folder will not be monitored."),
 			 GTK_MESSAGE_WARNING, NULL);
-		g_print ("homedir:%s\n",
-			 gnome_vfs_result_to_string (result));
+		g_print ("homedir:%s\n", error->message);
+		g_error_free (error);
+	}
+	else {
+		g_signal_connect (monitor_home,
+				  "changed",
+				  G_CALLBACK (contents_changed_cb),
+				  NULL);
 	}
 }
 
@@ -648,77 +737,193 @@ static void
 baobab_shutdown (void)
 {
 	g_free (baobab.label_scan);
-	g_string_free (baobab.last_scan_command, TRUE);
 
-	if (handle_mtab)
-		gnome_vfs_monitor_cancel (handle_mtab);
-	if (handle_home)
-		gnome_vfs_monitor_cancel (handle_home);
+	if (current_location)
+		g_object_unref (current_location);
+
+	if (monitor_vol)
+		g_object_unref (monitor_vol);
+	if (monitor_home)
+		g_file_monitor_cancel (monitor_home);
 
 	g_free (baobab.selected_path);
 
-	g_slist_foreach (baobab.bbExcludedDirs, (GFunc) g_free, NULL);
-	g_slist_free (baobab.bbExcludedDirs);
+	g_slist_foreach (baobab.excluded_locations, (GFunc) g_object_unref, NULL);
+	g_slist_free (baobab.excluded_locations);
 
 	if (baobab.gconf_client)
 		g_object_unref (baobab.gconf_client);
 }
 
+static void
+create_context_menu (void)
+{
+	ContextMenu *menu = NULL;
+
+	baobab.chart_menu = g_new0 (ContextMenu, 1);
+	menu = baobab.chart_menu;
+
+	menu->widget = gtk_menu_new ();
+		
+	menu->up_item = gtk_image_menu_item_new_with_label (_("Move to parent folder")); 
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu->up_item), 
+				       gtk_image_new_from_stock(GTK_STOCK_GO_UP, GTK_ICON_SIZE_MENU));
+		
+	menu->zoom_in_item = gtk_image_menu_item_new_with_label (_("Zoom in")) ;
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu->zoom_in_item), 
+				       gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
+        
+	menu->zoom_out_item = gtk_image_menu_item_new_with_label (_("Zoom out"));
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu->zoom_out_item), 
+				       gtk_image_new_from_stock(GTK_STOCK_REMOVE, GTK_ICON_SIZE_MENU));       
+		
+	menu->snapshot_item = gtk_image_menu_item_new_with_label (_("Save snapshot"));
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu->snapshot_item),
+									gtk_image_new_from_file (BAOBAB_PIX_DIR "shot.png"));
+
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu->widget),
+			       menu->up_item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu->widget),
+			       gtk_separator_menu_item_new ());
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu->widget),
+			       menu->zoom_in_item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu->widget),
+			       menu->zoom_out_item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu->widget),
+			       gtk_separator_menu_item_new ());
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu->widget),
+			       menu->snapshot_item);
+
+	/* connect signals */
+	g_signal_connect (menu->up_item, "activate",
+			  G_CALLBACK (on_move_upwards_cb), NULL);
+	g_signal_connect (menu->zoom_in_item, "activate",
+			  G_CALLBACK (on_zoom_in_cb), NULL);
+	g_signal_connect (menu->zoom_out_item, "activate",
+			  G_CALLBACK (on_zoom_out_cb), NULL);
+	g_signal_connect (menu->snapshot_item, "activate",
+					  G_CALLBACK (on_chart_snapshot_cb), NULL);
+
+	gtk_widget_show_all (menu->widget);
+}
+
 static void
 initialize_ringschart (void)
 {
-        GtkWidget *hpaned_main;
-        GtkWidget *ringschart_frame;
+	GtkWidget *hpaned_main;
+	GtkWidget *chart_frame;
+	GtkWidget *hbox1;
 
-        baobab.ringschart = GTK_WIDGET (baobab_ringschart_new ());
-        baobab_ringschart_set_model_with_columns (baobab.ringschart,
-                                                  GTK_TREE_MODEL (baobab.model),
-                                                  COL_DIR_NAME,
-                                                  COL_DIR_SIZE,
-                                                  COL_H_FULLPATH,
-                                                  COL_H_PERC,
-                                                  COL_H_ELEMENTS,
-                                                  NULL);
-        baobab_ringschart_set_init_depth (baobab.ringschart, 1);
+	chart_frame = gtk_frame_new (NULL);
+	gtk_frame_set_label_align (GTK_FRAME (chart_frame), 0.0, 0.0);
+	gtk_frame_set_shadow_type (GTK_FRAME (chart_frame), GTK_SHADOW_IN);
 
-        g_signal_connect (baobab.ringschart, "sector_activated",
-                          G_CALLBACK (on_rchart_sector_activated), NULL);
+	hpaned_main = glade_xml_get_widget (baobab.main_xml, "hpaned_main");
+	gtk_paned_pack2 (GTK_PANED (hpaned_main),
+			 chart_frame, TRUE, TRUE);
+	gtk_paned_set_position (GTK_PANED (hpaned_main), 480);
 
-        ringschart_frame = gtk_frame_new (NULL);
-        gtk_frame_set_label_align (GTK_FRAME (ringschart_frame), 0.0, 0.0);
-        gtk_frame_set_shadow_type (GTK_FRAME (ringschart_frame), GTK_SHADOW_IN);
+	baobab.chart_type_combo = gtk_combo_box_new_text ();
+	gtk_combo_box_append_text (GTK_COMBO_BOX (baobab.chart_type_combo), 
+				   _("View as Rings Chart"));
+	gtk_combo_box_append_text (GTK_COMBO_BOX (baobab.chart_type_combo), 
+				   _("View as Treemap Chart"));
+	gtk_combo_box_set_active (GTK_COMBO_BOX (baobab.chart_type_combo), 0);
+	gtk_widget_show (baobab.chart_type_combo);
+	g_signal_connect (baobab.chart_type_combo,
+			  "changed",
+			  G_CALLBACK (on_chart_type_change), NULL);
 
-        gtk_container_add (GTK_CONTAINER (ringschart_frame),
-                           baobab.ringschart);
+	hbox1 = glade_xml_get_widget (baobab.main_xml, "hbox1");
+	gtk_container_add (GTK_CONTAINER (hbox1), baobab.chart_type_combo);
+	gtk_box_set_spacing (GTK_BOX (hbox1), 50);
+	gtk_box_set_child_packing (GTK_BOX (hbox1),
+				   baobab.chart_type_combo,
+				   FALSE,
+				   TRUE,
+				   0, GTK_PACK_END);
 
-        hpaned_main = glade_xml_get_widget (baobab.main_xml, "hpaned_main");
-        gtk_paned_pack2 (GTK_PANED (hpaned_main),
-                         ringschart_frame, TRUE, TRUE);
-        gtk_paned_set_position (GTK_PANED (hpaned_main), 480);
+	/* Baobab's Treemap Chart */
+	baobab.treemap_chart = baobab_treemap_new ();
+	baobab_chart_set_model_with_columns (baobab.treemap_chart,
+					     GTK_TREE_MODEL (baobab.model),
+					     COL_DIR_NAME,
+					     COL_DIR_SIZE,
+					     COL_H_PARSENAME,
+					     COL_H_PERC,
+					     COL_H_ELEMENTS,
+					     NULL);
+	baobab_chart_set_max_depth (baobab.treemap_chart, 1);
+	g_signal_connect (baobab.treemap_chart, "item_activated",
+					G_CALLBACK (on_chart_item_activated), NULL);
+	g_signal_connect (baobab.treemap_chart, "button-release-event", 
+					G_CALLBACK (on_chart_button_release), NULL);
+	gtk_widget_show (baobab.treemap_chart);
+	/* Ends Baobab's Treemap Chart */
 
-        baobab_ringschart_draw_center (baobab.ringschart, TRUE);
+	/* Baobab's Rings Chart */
+	baobab.rings_chart = (GtkWidget *) baobab_ringschart_new ();
+	baobab_chart_set_model_with_columns (baobab.rings_chart,
+					     GTK_TREE_MODEL (baobab.model),
+					     COL_DIR_NAME,
+					     COL_DIR_SIZE,
+					     COL_H_PARSENAME,
+					     COL_H_PERC,
+					     COL_H_ELEMENTS,
+					     NULL);
+	baobab_ringschart_set_subfoldertips_enabled (baobab.rings_chart,
+						     gconf_client_get_bool (baobab.gconf_client,
+									    BAOBAB_SUBFLSTIPS_VISIBLE_KEY,
+									    NULL));
+	baobab_chart_set_max_depth (baobab.rings_chart, 1);
+	g_signal_connect (baobab.rings_chart, "item_activated",
+					G_CALLBACK (on_chart_item_activated), NULL);
+	g_signal_connect (baobab.rings_chart, "button-release-event", 
+					G_CALLBACK (on_chart_button_release), NULL);
+	gtk_widget_show (baobab.rings_chart);
+	/* Ends Baobab's Treemap Chart */
 
-        gtk_widget_show_all (ringschart_frame);
+	baobab.current_chart = baobab.rings_chart;
+
+	g_object_ref_sink (baobab.treemap_chart);
+	baobab_chart_freeze_updates (baobab.treemap_chart);
+
+	gtk_container_add (GTK_CONTAINER (chart_frame),
+			   baobab.current_chart);
+	gtk_widget_show_all (chart_frame);
+
+	create_context_menu ();
+}
+
+static gboolean
+start_proc_on_command_line (GFile *file)
+{
+	baobab_scan_location (file);
+
+	return FALSE;
 }
 
 int
 main (int argc, char *argv[])
 {
 	GnomeProgram *program;
+	GOptionContext *context;
 
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
+	context = g_option_context_new (NULL);
+
 	program = gnome_program_init ("baobab", VERSION,
 				      LIBGNOMEUI_MODULE,
 				      argc, argv,
+				      GNOME_PARAM_GOPTION_CONTEXT, context,
 				      GNOME_PARAM_APP_DATADIR,
 				      BAOBAB_DATA_DIR, NULL);
 
 	g_set_application_name ("Baobab");
 
-	g_assert (gnome_vfs_init ());
 	gnome_authentication_manager_init ();
 	glibtop_init ();
 
@@ -726,18 +931,20 @@ main (int argc, char *argv[])
 
 	baobab_init ();
 
+	g_noactivescans = TRUE;
+	check_menu_sens (FALSE);
 	baobab_get_filesystem (&g_fs);
 	if (g_fs.total == 0) {
 		GtkWidget *dialog = gtk_message_dialog_new (NULL,
                                   GTK_DIALOG_DESTROY_WITH_PARENT,
                                   GTK_MESSAGE_ERROR,
                                   GTK_BUTTONS_CLOSE,
-                                  "Could not detect any mount point.");
+                                  _("Could not detect any mount point."));
                 gtk_message_dialog_format_secondary_text
                                                         (GTK_MESSAGE_DIALOG(dialog),
-                                                         "Without mount points disk usage cannot be analyzed.");
- 		gtk_dialog_run (GTK_DIALOG (dialog));
- 		gtk_widget_destroy (dialog);
+                                                         _("Without mount points disk usage cannot be analyzed."));
+		gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
 		goto closing;
 	}
 
@@ -750,7 +957,7 @@ main (int argc, char *argv[])
 
 	baobab.tree_view = create_directory_treeview ();
 
-	set_glade_widget_sens("menurescan",FALSE);
+	set_glade_widget_sens ("menurescan",FALSE);
 
 	/* set allocated space checkbox */
 	gtk_check_menu_item_set_active ((GtkCheckMenuItem *)
@@ -759,7 +966,6 @@ main (int argc, char *argv[])
 							    "ck_allocated"),
 				      baobab.show_allocated);
 
-	set_glade_widget_sens("menu_treemap",FALSE);
 	gtk_widget_show (baobab.window);
 
 	first_row ();
@@ -770,10 +976,16 @@ main (int argc, char *argv[])
 
 	/* commandline */
 	if (argc > 1) {
-		gchar *uri_shell;
-		uri_shell = gnome_vfs_make_uri_from_shell_arg (argv[1]);
-		start_proc_on_dir (uri_shell);
-		g_free (uri_shell);
+		GFile *file;
+
+		file = g_file_new_for_commandline_arg (argv[1]);
+
+		/* start processing the dir specified on the
+		 * command line as soon as we enter the main loop */
+		g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
+		                 (GSourceFunc) start_proc_on_command_line,
+		                 file,
+		                 (GDestroyNotify) g_object_unref);
 	}
 
 	gtk_main ();
@@ -782,7 +994,6 @@ main (int argc, char *argv[])
 	baobab_shutdown ();
 
 	glibtop_close ();
-	gnome_vfs_shutdown ();
 
 	g_object_unref (program);
 
