@@ -1,10 +1,29 @@
+/* screenshot-save.c - image saving functions for GNOME Screenshot
+ *
+ * Copyright (C) 2001-2006  Jonathan Blandford <jrb@alum.mit.edu>
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public
+ * License along with this program; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ */
+
 #include <config.h>
-#include <gnome.h>
+#include <stdlib.h>
+#include <unistd.h>
 #include <errno.h>
 #include <signal.h>
 #include <string.h>
 #include <glib/gi18n.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 
 #include "screenshot-save.h"
 
@@ -44,13 +63,13 @@ clean_up_temporary_dir (gboolean gui_on_error)
 	  dialog = gtk_message_dialog_new (NULL, 0,
 					   GTK_MESSAGE_ERROR,
 					   GTK_BUTTONS_OK,
-					   message);
+					   "%s", message);
 	  gtk_dialog_run (GTK_DIALOG (dialog));
 	  gtk_widget_destroy (dialog);
 	}
       else
 	{
-	  g_warning (message);
+	  g_warning ("%s", message);
 	}
       g_free (message);
     }
@@ -102,7 +121,7 @@ read_pipe_from_child (GIOChannel   *source,
 	  dialog = gtk_message_dialog_new (NULL, 0,
 					   GTK_MESSAGE_ERROR,
 					   GTK_BUTTONS_OK,
-					   message);
+					   "%s", message);
 	  gtk_dialog_run (GTK_DIALOG (dialog));
 	  gtk_widget_destroy (dialog);
 	  exit (1);
@@ -123,17 +142,16 @@ make_temp_directory (void)
   i = 0;
   do
     {
-      gchar *tmp_dir = g_strdup_printf ("%u-%d",
+      gchar *tmp_dir = g_strdup_printf ("gnome-screenshot.%u.%d",
                                         (unsigned int) getpid (),
                                         i++);
       
       dir_name = g_build_filename (g_get_tmp_dir (),
-                                   "gnome-screenshot",
                                    tmp_dir,
                                    NULL);
       g_free (tmp_dir);
 
-      result = g_mkdir_with_parents (dir_name, 0700);
+      result = g_mkdir_with_parents (dir_name, 0777);
       if (result < 0)
         {
           g_free (dir_name);
@@ -194,7 +212,7 @@ screenshot_save_start (GdkPixbuf    *pixbuf,
 
       if (! gdk_pixbuf_save (pixbuf, tmp_filename,
 			     "png", &error,
-			     "tEXt::CREATOR", "gnome-panel-screenshot",
+			     "tEXt::Software", "gnome-screenshot",
 			     NULL))
 	{
 	  if (error && error->message)
@@ -253,7 +271,9 @@ screenshot_sanitize_filename (const char *filename)
   g_assert (filename);
   g_assert (g_utf8_validate (filename, -1, NULL));
 
-  retval = gnome_vfs_escape_host_and_path_string (filename);
+  retval = g_uri_escape_string (filename,
+                                NULL,
+                                TRUE);
 
   for (p = retval; *p != '\000'; p = g_utf8_next_char (p))
     {
